/*
 * StringTransferHandler.java is used by the 1.4
 * ExtendedDnDDemo.java example.
 */
package dnd;

import java.awt.datatransfer.*;
import javax.swing.*;
import java.io.IOException;

public abstract class StringTransferHandler extends TransferHandler {
/** At the heart of the data transfer mechanism is the TransferHandler class.
* A TransferHandler provides an easy mechanism for transferring data to and
* from a JComponent. The data to be transferred is bundled up into an object 
* that implements the Transferable interface. The components in the data 
* transfer support table are provided with default transfer handlers, but a 
* transfer handler can be created and installed on any component using the 
* JComponent method setTransferHandler.*/
    
/** An abstract
* subclass of TransferHandler, StringTransferHandler, defines three abstract
* methods for importing and exporting strings: exportString, importString, and 
* cleanup. StringTransferHandler also overrides the standard TransferHandler
* methods: importData and canImport are required to import data; 
* getSourceActions, createTransferable, and exportDone are for export. 
* (Note that exportDone may not be necessary if you only implement Copy and, 
* therefore, do not need to remove data from the source as you would for a 
* Move.) The abstract methods importString, exportString, and cleanup are 
* called by the importData, createTransferable, and exportDone methods, 
* respectively. */
    
    protected abstract String exportString(JComponent c);
    protected abstract void importString(JComponent c, String str);
    protected abstract void cleanup(JComponent c, boolean remove);
    
    protected Transferable createTransferable(JComponent c) {
        return new StringSelection(exportString(c));
    }
    
    public int getSourceActions(JComponent c) {
        /**el estado retornar copia significa que el objeto permanece en el destino y en 
        * el origen, otro posible valor es CUT, pero no serviria para el dnd que necesito*/        
        return this.COPY;
    }
    
    /**la funcion metodo importData toma dos valores*/
    public boolean importData(JComponent c, Transferable t) {
        
       /**If you cannot use the property name form of the TransferHandler
       * constructor, the DataFlavor class allows you to specify the 
       * content-type of your data for both your TransferHandler and, if 
       * necessary, your Transferable. Three flavor types are predefined 
       * for you:

       * imageFlavor represents data in the java.awt.Image format
       * stringFlavor represents data in the most basic form of text  java.lang.String
       * javaFileListFlavor represents java.io.File objects in a java.util.List format 
        
       * To create a data flavor for an integer array:
       * new DataFlavor(int[].class, "Integer Array");

       * Transferring the data in this manner uses Object serialization, 
       * so the class you use to transfer the data must implement the 
       * Serializable interface, as must anything that is serialized with it. 
       * If not everything is Serializable, you'll see a 
       * NotSerializableException during drop or copy to the clipboard.*/
        
        if (canImport(c, t.getTransferDataFlavors())) {
            try {
                String str = (String)t.getTransferData(DataFlavor.stringFlavor);
                importString(c, str);
                return true;
            } catch (UnsupportedFlavorException ufe) {
            } catch (IOException ioe) {
            }
        }

        return false;
    }
    
    protected void exportDone(JComponent c, Transferable data, int action) {
        cleanup(c, action == MOVE);
    }
    
    public boolean canImport(JComponent c, DataFlavor[] flavors) {
        for (int i = 0; i < flavors.length; i++) {
            if (DataFlavor.stringFlavor.equals(flavors[i])) {
                return true;
            }
        }
        return false;
    }
}